# gwschedule.py - handles scheduling of jobs and the gumstix

import datetime
import os
import ConfigParser

import gwutil
import Gumsense
import batvolts

BOOT_HOUR = 11
BOOT_MINUTE = 58

CRITICAL = 0
LOW = 1
OK = 2
HIGH = 3

INFO_FILE = "/home/root/info"
OVERIDE_FILE = "/home/root/overide_state"
STATE_DIR = "/home/root/states"

def get_last_run_time():
    """Reads the last time of execution from a file
    Returns a datetime object"""
    f = open( INFO_FILE, "r" )
    t = None
    for l in f:
        l = l.strip()
        if len(l) > 0:
            t = datetime.datetime.fromtimestamp(int(l))
            break
    f.close()
    return t

def set_last_run_time():
    ifile = open(INFO_FILE, "w")
    ifile.write( str( gwutil.totimestamp(datetime.datetime.utcnow()) ) + "\n" )
    ifile.close()

def set_next_wakeup(gum):
    now = datetime.datetime.utcnow()
    run_at = datetime.datetime(now.year,now.month,now.day, BOOT_HOUR, BOOT_MINUTE)
    if now > run_at:
        tomorrow = datetime.datetime.utcnow() + datetime.timedelta(days=1)
        run_at = datetime.datetime(tomorrow.year,
                                   tomorrow.month,
                                   tomorrow.day,
                                   BOOT_HOUR,BOOT_MINUTE)

    wake_up_unix = gwutil.totimestamp(run_at)
    print "Next wakeup: %s (%i)" % (str(run_at), wake_up_unix)

    # Tell the Gumsense to next wake up
    gum.wake = run_at

def get_state():
    state = batvolts.get_state()
    return state

def force_state(gum, new_state):
    print "Setting job schedule to state %i"%new_state
    #clear jobs
    gum.jobs.pause()
    for i in range(0,len(gum.jobs)):
        gum.jobs.clear(i)
    
    #read state file
    config = ConfigParser.RawConfigParser()
    config.read("%s/%i"%(STATE_DIR,new_state))
    try:
        for section in config.sections():
            job = getattr(gum.jobs, section)
            valid_ops =['interval', 'start_time', 'power_mask','power_up_time', 'sensors']
            interval = None
            start_time = None
            power_mask = None
            power_up_time = None
            sensors = None
            start_time_set = False
            for k,v in config.items(section):
                if k not in valid_ops:
                    print "Unknown option %s, ignoring"%k
                else:
                    if k == 'interval':
                        t = eval('datetime.timedelta(%s)'%v)
                        interval = t
                    elif k == 'start_time':
                        t = datetime.datetime.strptime(v, "%H:%M:%S")
                        now = datetime.datetime.utcnow()
                        n = datetime.datetime(now.year, now.month, now.day, t.hour, t.minute, t.second)
                        if interval is not None:
                            n = __next_time(n, interval)
                            start_time = gwutil.totimestamp(n)
                        else:
                            start_time_set = True
                            start_time = n
                    elif k == 'power_mask':
                        power_mask = int(v)
                    elif k == 'power_up_time':
                        t = eval('datetime.timedelta(%s)'%v)
                        power_up_time = t
                    elif k == 'sensors':
                        sensors = []
                        config_sensors = eval(v)
                        for sensor in config_sensors:
                            try:
                                s = getattr(gum.sensors,sensor)
                                sensors.append(s)
                            except Exception:
                                print "Unknown sensor %s"%sensor
		 
                        if len(sensors) == 0:
                            sensors = None
            
            if start_time_set:
                n = __next_time(start_time,interval)
                start_time = gwutil.totimestamp(n)
            
            if interval is not None:
                interval = (interval.days * ( 3600 * 24)) + interval.seconds
            if power_up_time is not None:
                p = power_up_time
                power_up_time = (p.days * (3600 * 24)) + p.seconds
	    try:
                job.modify(interval=interval, 
                               next_time=start_time, 
                               power_up_time=power_up_time, 
                               power_mask=power_mask,
                               sensors=sensors)
            except Exception:
                print "Setting job failed"

    except Exception, e:
        print "%s"%e
        print "No such job called %s in gumsense.conf"%section
        
    gum.jobs.resume()

def set_local_state(gum):
    previous_state = get_state()
    state, avg = batvolts.set_state(gum)
    if state != previous_state or state == CRITICAL:
        force_state(gum, state)

def set_state(gum):
    current_state = get_state()
    f = open(OVERIDE_FILE,"r")
    try:
        state = f.readline()
        if state is None or state == '':
            print "No overide state available"
            return
        if int(state[0]) in range(CRITICAL, HIGH+1):
            new_state = int(state[0])
            if new_state is not current_state:
                if new_state < current_state:
                    force_set(gum,new_state)
                else:
                    print "Overide state %i > local state %i"%(new_state, current_state)
                    print "Keeping local state job schedule"
            else:
                print "Overide state %i == local state %i"%(new_state, current_state)
                print "No changes to be made to job schedule"
        else:
            raise Exception('Unknown state %i'%int(state[0]))
    except Exception, e:
        print e
	print "Failed to set state to %i using local state %i"%(int(state[0]), current_state)
    finally:
        f.close()
        os.remove(OVERIDE_FILE)

def __next_time(n, interval):
    # n is the time set in start_time
    # interval is the interval between jobs
    # We want to set the next_time to be some offset in the future.
    # if days takes you over next, set start time to be +#days+start_time
    now = datetime.datetime.utcnow()
    new = n
    while new < now:
        if interval.days > 0:
            new = new + datetime.timedelta(days=interval.days)
            break
        else:
            new = new + interval

    return new
